@@ -1,4 +1,5 @@
 #Copyright (C) 2008 Codethink Ltd
+#Copyright (C) 2010 Novell, Inc.
 
 #This library is free software; you can redistribute it and/or
 #modify it under the terms of the GNU Lesser General Public
@@ -12,8 +13,7 @@
 #along with this program; if not, write to the Free Software
 #Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-from interfaces import *
-from accessible import Accessible
+from gi.repository import Atspi
 
 __all__ = [
            "Document",
@@ -21,7 +21,7 @@ __all__ = [
 
 #------------------------------------------------------------------------------
 
-class Document(Accessible):
+class Document:
         """
         Primarily a 'tagging' interface which indicates the start of
         document content in the Accessibility hierarchy. Accessible objects
@@ -32,6 +32,9 @@ class Document(Accessible):
         as well.
         """
 
+        def __init__(self, obj):
+                self.obj = obj
+
         def getAttributeValue(self, key):
                 """
                 Gets the value of a single attribute, if specified for the document
@@ -43,8 +46,7 @@ class Document(Accessible):
                 attribute, or an empty string if the attribute is unspecified
                 for the object.
                 """
-                func = self.get_dbus_method("GetAttributeValue", dbus_interface=ATSPI_DOCUMENT)
-                return func(key)
+                return Atspi.Document.get_attribute_value(self.obj, key)
 
         def getAttributes(self):
                 """
@@ -54,8 +56,8 @@ class Document(Accessible):
                 @return an AttributeSet containing the attributes of the document,
                 as name-value pairs.
                 """
-                func = self.get_dbus_method("GetAttributes", dbus_interface=ATSPI_DOCUMENT)
-                return [key + ':' + value for key, value in func().iteritems()]
+                ret = Atspi.Document.get_attributes(self.obj)
+                return [key + ':' + value for key, value in ret.iteritems()]
 
         def getLocale(self):
                 """
@@ -64,7 +66,6 @@ class Document(Accessible):
                 @return a string compliant with the POSIX standard for locale
                 description.
                 """
-                func = self.get_dbus_method("GetLocale", dbus_interface=ATSPI_DOCUMENT)
-                return func()
+                return Atspi.Document.get_locale(self.obj)
 
 #END----------------------------------------------------------------------------
